#include "XMLSerializer.h"


#ifndef _WIN32
    #include <inttypes.h>
#endif


XMLSerializer::XMLSerializer(FILE *a_pFile)
:    m_pFile(a_pFile)
,    m_iIndentLevel(0)
{
    ;
}


bool XMLSerializer::IsWritingToObject() const
{
    return false;
}


bool XMLSerializer::Serialize(bool &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%s\" />\n", m_iIndentLevel, "", a_Name, "bool", a_Value ? "True" : "False");
    return true;
}


bool XMLSerializer::Serialize(char &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%c\" />\n", m_iIndentLevel, "", a_Name, "char", a_Value);
    return true;
}


bool XMLSerializer::Serialize(int8_t &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%hhd\" />\n", m_iIndentLevel, "", a_Name, "int8_t", a_Value);
    return true;
}


bool XMLSerializer::Serialize(int16_t &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%hd\" />\n", m_iIndentLevel, "", a_Name, "int16_t", a_Value);
    return true;
}


bool XMLSerializer::Serialize(int32_t &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%d\" />\n", m_iIndentLevel, "", a_Name, "int32_t", a_Value);
    return true;
}


bool XMLSerializer::Serialize(int64_t &a_Value, const char *a_Name)
{
# if __WORDSIZE == 64
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%ld\" />\n", m_iIndentLevel, "", a_Name, "int64_t", a_Value);
#else
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%lld\" />\n", m_iIndentLevel, "", a_Name, "int64_t", a_Value);
#endif

    return true;
}


bool XMLSerializer::Serialize(uint8_t &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%hhu\" />\n", m_iIndentLevel, "", a_Name, "uint8_t", a_Value);
    return true;
}


bool XMLSerializer::Serialize(uint16_t &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%hu\" />\n", m_iIndentLevel, "", a_Name, "uint16_t", a_Value);
    return true;
}


bool XMLSerializer::Serialize(uint32_t &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%u\" />\n", m_iIndentLevel, "", a_Name, "uint32_t", a_Value);
    return true;
}


bool XMLSerializer::Serialize(uint64_t &a_Value, const char *a_Name)
{
# if __WORDSIZE == 64
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%lu\" />\n", m_iIndentLevel, "", a_Name, "uint64_t", a_Value);
#else
    fprintf(m_pFile, "%*s<Member Name=\"%s\" Type=\"%s\" Value=\"%llu\" />\n", m_iIndentLevel, "", a_Name, "uint64_t", a_Value);
#endif

    return true;
}


bool XMLSerializer::Serialize(ISerializable &a_Value, const char *a_Name)
{
    fprintf(m_pFile, "%*s<Structure Name=\"%s\" >\n", m_iIndentLevel, "", a_Name);
    m_iIndentLevel += 3;

    if (!a_Value.Serialize(*this, a_Name))
    {
        return false;
    }

    m_iIndentLevel -= 3;
    fprintf(m_pFile, "%*s</Structure>\n", m_iIndentLevel, "");

    return true;
}
