#ifndef BASE_SERVICE_H
#define BASE_SERVICE_H


#include "IService.h"
#include "ISerializer.h"
#include "ResultCodes.h"
#include "TimeoutQueue.h"
#include "StrongTypedef.h"
#include "PacketMetadata.h"
#include "NetworkInputSerializer.h"
#include "NetworkOutputSerializer.h"
#include <map>


#include "PacketHeader.autogen.h"
#include "CoroutineHeader.autogen.h"


// Some maximum packet size
static const uint32_t MAX_PACKET_SIZE = 4096;
typedef uint8_t PacketBuffer[MAX_PACKET_SIZE];


struct PacketHeader
:   public ISerializable
{
    typedef SizedType< RequiredBytesForValue<EPacketType_COUNT>::Value >::Type SerializePacketType;

    ServiceId           m_SendingServiceId;
    SerializePacketType m_PacketType;

    PacketHeader(ServiceId a_SendingServiceId, EPacketType a_PacketType)
    :   m_SendingServiceId(a_SendingServiceId)
    ,   m_PacketType(SerializePacketType(a_PacketType))
    {
        ;
    }

    virtual const char *GetSerializableClassName() const
    {
        return "PacketHeader";
    }

    virtual bool Serialize(ISerializer &a_Serializer, const char *a_Name)
    {
        return a_Serializer.Serialize(m_SendingServiceId, "SendingServiceId") && 
               a_Serializer.Serialize(m_PacketType,       "PacketType");
    }
};


/**
 * @class ServiceList
 * 
 * @brief assumes the role of a message router and service discovery system
 */
class ServiceList
{
public:

    void RegisterService(ServiceId a_ServiceId, IService *a_pService);

    IService *GetService(ServiceId a_ServiceId) const;

private:

    typedef std::map<ServiceId, IService *> ServiceMap;

    ServiceMap m_ServiceMap;

};


class BaseService
:   public IService
{
public:

    BaseService(ServiceId a_ServiceId, ServiceList &a_ServiceList);

    virtual ServiceId GetServiceId() const;

    virtual bool SendTo(ServiceId a_DstServiceId, IPacket &a_Packet);

    virtual TimeoutId RegisterItem(ITimeoutItem *a_pItem);

    virtual ITimeoutItem *RetrieveItem(TimeoutId a_TimeoutId);

    virtual void RemoveItem(TimeoutId a_TimeoutId);

    virtual void Update();

    virtual bool InternalReceiveFrom(const uint8_t *a_pData, uint32_t a_iNumBytes);

private:

    struct QueuedMessage
    {
        ServiceId                m_SendingServiceId;
        std::shared_ptr<IPacket> m_Message;
    };

    typedef std::vector<QueuedMessage> MessageBuffer;

    ServiceId     m_ServiceId;

    ServiceList  &m_ServiceList;

    TimeoutQueue  m_TimeoutQueue;

    MessageBuffer m_MessageBuffer;

};


#endif
