#include "CommonUtils.h"
#include <stdarg.h>
#include <stdio.h>


#ifdef _WIN32
    #define WIN32_LEAN_AND_MEAN
    #include <windows.h>
#endif


void WriteDebugString(const char *a_Format, ...)
{
    va_list args;
    va_start(args, a_Format);

    static const uint32_t MAX_OUTPUT_STRING_SIZE = 1024;
    char buffer[MAX_OUTPUT_STRING_SIZE];

#ifdef _WIN32
    vsnprintf_s(buffer, MAX_OUTPUT_STRING_SIZE, _TRUNCATE, a_Format, args);
#else
    vsnprintf(buffer, MAX_OUTPUT_STRING_SIZE, a_Format, args);
#endif

    va_end(args);

#ifdef _WIN32
    OutputDebugStringA(buffer);
#else
    fprintf(stdout, "%s", buffer);
#endif
}


#ifdef _WIN32
    uint64_t GetCurrentNanoseconds()
    {
        static bool          s_bTimeInitialized = false;
        static uint64_t      s_iPerfFrequency   = 0;
        static LARGE_INTEGER s_iStartTime;

        if (!s_bTimeInitialized)
        {
            LARGE_INTEGER iTicksPerSecond;
            QueryPerformanceCounter(&s_iStartTime);
            QueryPerformanceFrequency(&iTicksPerSecond);

            s_iPerfFrequency = iTicksPerSecond.QuadPart;

            s_bTimeInitialized = true;
        }

        LARGE_INTEGER iTick;

        QueryPerformanceCounter(&iTick);

        return (uint64_t)((iTick.QuadPart - s_iStartTime.QuadPart) * 1000000000) / s_iPerfFrequency;
    }
#elif defined __APPLE__
    #include <mach/mach.h>
    #include <mach/mach_time.h>

    // UNTESTED!
    uint64_t GetCurrentNanoseconds()
    {
        static bool                      s_bTimeInitialized = false;
        static mach_timebase_info_data_t s_iPerfFrequency;
        static uint64_t                  s_iStartTime;
        
        if (!s_bTimeInitialized)
        {
            mach_timebase_info(&s_iPerfFrequency);
            s_iStartTime = ((mach_absolute_time() * s_iPerfFrequency.numer) / s_iPerfFrequency.denom);
            s_bTimeInitialized = true;
        }

        uint64_t iCurrentTime = ((mach_absolute_time() * s_iPerfFrequency.numer) / s_iPerfFrequency.denom);

        return iCurrentTime - s_iStartTime;
    }
#else
   #include <time.h>

    // UNTESTED!
   uint64_t GetCurrentNanoseconds()
   {
        static bool     s_bTimeInitialized    = false;
        static timespec s_iStartTime;

        if (!s_bTimeInitialized)
        {
            clock_gettime(CLOCK_REALTIME, &s_iStartTime);
            s_bTimeInitialized = true;
        }

        timespec currTime;

        clock_gettime(CLOCK_REALTIME, &currTime);

        uint64_t iSeconds     = 0;
        uint64_t iNanoseconds = 0;

        if ((currTime.tv_nsec - s_iStartTime.tv_nsec) < 0)
        {
            iSeconds     = uint64_t(currTime.tv_sec  - s_iStartTime.tv_sec  - 1);
            iNanoseconds = uint64_t(currTime.tv_nsec - s_iStartTime.tv_nsec + 1000000000);
        }
        else
        {
            iSeconds     = uint64_t(currTime.tv_sec  - s_iStartTime.tv_sec);
            iNanoseconds = uint64_t(currTime.tv_nsec - s_iStartTime.tv_nsec);
        }

        return (iSeconds * 1000000000LLU) + iNanoseconds;
   }
#endif
