#ifndef COMMON_UTILS_H
#define COMMON_UTILS_H


#include <stdint.h>


/**
 * Writes an output string to some log somewhere, OutputDebugString on windows, stdout everywhere else
 * 
 * @param a_String the string to write
 */
void WriteDebugString(const char *a_Format, ...);


/**
 * Returns nanoseconds since program start
 * 
 * @return nanoseconds since program start
 */
uint64_t GetCurrentNanoseconds();


/**
 * Returns milliseconds since program start
 * 
 * @return milliseconds since program start
 */
uint32_t GetCurrentMilliseconds();


/**
 * @struct RequiredBytesForBitfield
 * 
 * @brief calculates at compile-time the number of bytes required to represent the given set of input bits
 */
template <uint64_t MAX_VALUE>
struct RequiredBytesForBitfield
{
    enum
    {
        Value = (MAX_VALUE > 64) ? 0xFFFFFFFF // Failure case, just need a value that does not resolve to a valid type
              : (MAX_VALUE > 32) ? 8 
              : (MAX_VALUE > 16) ? 4 
              : (MAX_VALUE >  8) ? 2 
              : 1
    };
};


/**
 * @struct RequiredBytesForValue
 * 
 * @brief calculates at compile-time the number of bytes required to represent the given max value
 */
template <uint64_t MAX_VALUE>
struct RequiredBytesForValue
{
    enum
    {
        Value = (MAX_VALUE > 0xFFFFFFFFFFFFFFFF) ? 0xFFFFFFFF // Failure case, just need a value that does not resolve to a valid type
              : (MAX_VALUE > 0x00000000FFFFFFFF) ? 8 
              : (MAX_VALUE > 0x000000000000FFFF) ? 4 
              : (MAX_VALUE > 0x00000000000000FF) ? 2 
              : 1
    };
};


template <uint32_t BYTE_COUNT> struct SizedType { };
template <> struct SizedType<8> { typedef uint64_t Type; };
template <> struct SizedType<4> { typedef uint32_t Type; };
template <> struct SizedType<2> { typedef uint16_t Type; };
template <> struct SizedType<1> { typedef  uint8_t Type; };


#include "CommonUtils.inl"


#endif
