#ifndef NETWORK_INPUT_SERIALIZER_H
#define NETWORK_INPUT_SERIALIZER_H


#include "ISerializer.h"


/**
 * @class NetworkInputSerializer
 * 
 * @brief Simple example of a binary network serialize reader, definitely not production code...
 */
class NetworkInputSerializer
:    public ISerializer
{
public:

    /**
     * Constructor
     * 
     * @param a_pBuffer         input buffer to write to
     * @param a_iBufferCapacity capacity of the buffer in bytes
     */
    NetworkInputSerializer(uint8_t *a_pBuffer, uint32_t a_iBufferCapacity);

    /**
     * Copies the provided bytes into the serialization output buffer
     * 
     * @param a_pData     pointer to the data buffer to copy
     * @param a_iDataSize size of the data in bytes
     * 
     * @return boolean true on success, false if there was insufficient space to store all the data
     */
    bool CopyToBuffer(const uint8_t *a_pData, uint32_t a_iDataSize);

    /**
     * Returns a pointer to the serialization output buffer
     * 
     * @return pointer to the serialization output buffer
     */
    const uint8_t *GetBuffer() const;

    /**
     * Returns the size of the data contained in the serialization output buffer in bytes
     * 
     * @return size of the data contained in the serialization output buffer in bytes
     */
    uint32_t GetBufferSize() const;

    // ISerializer interfaces
    virtual bool IsWritingToObject() const;
    virtual bool Serialize(    bool &a_Value, const char *a_Name);
    virtual bool Serialize(    char &a_Value, const char *a_Name);
    virtual bool Serialize(  int8_t &a_Value, const char *a_Name);
    virtual bool Serialize( int16_t &a_Value, const char *a_Name);
    virtual bool Serialize( int32_t &a_Value, const char *a_Name);
    virtual bool Serialize( int64_t &a_Value, const char *a_Name);
    virtual bool Serialize( uint8_t &a_Value, const char *a_Name);
    virtual bool Serialize(uint16_t &a_Value, const char *a_Name);
    virtual bool Serialize(uint32_t &a_Value, const char *a_Name);
    virtual bool Serialize(uint64_t &a_Value, const char *a_Name);
    virtual bool Serialize(ISerializable &a_Value, const char *a_Name);
    // ISerializer interfaces

private:

    bool SerializeBytes(const uint8_t *a_pData, uint32_t a_iCount);

    bool     m_bSerializationFailed;

    uint32_t m_iBufferSize;

    const uint8_t *m_pBuffer;

    const uint32_t m_iBufferCapacity;

};


#endif
