#include "NetworkOutputSerializer.h"
#include <string.h>


#ifdef _WIN32
    #include <Winsock2.h>
#elif defined __APPLE_CC__
    #include <Machine/endian.h>
#else
    #include <endian.h>
    #include <string.h>
    #include <arpa/inet.h>
#endif


// It works, but many don't like this
// Easy enough to add a run-time check instead if you really want, but my goal here is simply minimal code
#if __BYTE_ORDER__ == __LITTLE_ENDIAN
    #define ntohll(X) (((uint64_t)ntohl((uint32_t)(X))) << 32 | (uint64_t)ntohl((uint32_t)(X >> 32)))
#else
    #define ntohll(X) X
#endif


NetworkOutputSerializer::NetworkOutputSerializer(const uint8_t *a_pBuffer, uint32_t a_iBufferCapacity)
:   m_bSerializationFailed(false) 
,   m_iBufferPosition(0)
,   m_pBuffer(a_pBuffer)
,   m_iBufferCapacity(a_iBufferCapacity)
{
    ;
}


uint32_t NetworkOutputSerializer::GetUnreadSize() const
{
    return (m_iBufferCapacity - m_iBufferPosition);
}


bool NetworkOutputSerializer::IsWritingToObject() const
{
    return false;
}


bool NetworkOutputSerializer::Serialize(bool &a_Value, const char *a_Name)
{
    uint8_t iValue = 0;
    bool bResult = SerializeBytes((uint8_t *)&iValue, sizeof(iValue));
    a_Value = (iValue > 0);
    return bResult;
}


bool NetworkOutputSerializer::Serialize(char &a_Value, const char *a_Name)
{
    return SerializeBytes((uint8_t *)&a_Value, sizeof(char));
}


bool NetworkOutputSerializer::Serialize(int8_t &a_Value, const char *a_Name)
{
    return SerializeBytes((uint8_t *)&a_Value, sizeof(int8_t));
}


bool NetworkOutputSerializer::Serialize(int16_t &a_Value, const char *a_Name)
{
    int16_t iNetworkOrder = 0;
    bool bResult = SerializeBytes((uint8_t *)&iNetworkOrder, sizeof(int16_t));
    a_Value = ntohs(iNetworkOrder);
    return bResult;
}


bool NetworkOutputSerializer::Serialize(int32_t &a_Value, const char *a_Name)
{
    int32_t iNetworkOrder = 0;
    bool bResult = SerializeBytes((uint8_t *)&iNetworkOrder, sizeof(int32_t));
    a_Value = ntohl(iNetworkOrder);
    return bResult;
}


bool NetworkOutputSerializer::Serialize(int64_t &a_Value, const char *a_Name)
{
    int64_t iNetworkOrder = 0;
    bool bResult = SerializeBytes((uint8_t *)&iNetworkOrder, sizeof(int64_t));
    a_Value = ntohll(iNetworkOrder);
    return bResult;
}


bool NetworkOutputSerializer::Serialize(uint8_t &a_Value, const char *a_Name)
{
    return SerializeBytes((uint8_t *)&a_Value, sizeof(uint8_t));
}


bool NetworkOutputSerializer::Serialize(uint16_t &a_Value, const char *a_Name)
{
    uint16_t iNetworkOrder = 0;
    bool bResult = SerializeBytes((uint8_t *)&iNetworkOrder, sizeof(uint16_t));
    a_Value = ntohs(iNetworkOrder);
    return bResult;
}


bool NetworkOutputSerializer::Serialize(uint32_t &a_Value, const char *a_Name)
{
    uint32_t iNetworkOrder = 0;
    bool bResult = SerializeBytes((uint8_t *)&iNetworkOrder, sizeof(uint32_t));
    a_Value = ntohl(iNetworkOrder);
    return bResult;
}


bool NetworkOutputSerializer::Serialize(uint64_t &a_Value, const char *a_Name)
{
    uint64_t iNetworkOrder = 0;
    bool bResult = SerializeBytes((uint8_t *)&iNetworkOrder, sizeof(uint64_t));
    a_Value = ntohll(iNetworkOrder);
    return bResult;
}


bool NetworkOutputSerializer::Serialize(ISerializable &a_Value, const char *a_Name)
{
    return a_Value.Serialize(*this, a_Name);
}


bool NetworkOutputSerializer::SerializeBytes(uint8_t *a_pData, uint32_t a_iCount)
{
    const uint32_t iCurrSize = m_iBufferPosition;
    const uint32_t iNextSize = m_iBufferPosition + a_iCount;

    if (m_bSerializationFailed || (iNextSize > m_iBufferCapacity))
    {
        // Keep the failed boolean as a paranoid check
        // If everyone honours the serialize boolean return values and terminates on fail, then this check is redundant
        // Otherwise we might fail to serialize a large value, and then succeed a smaller value
        m_bSerializationFailed = true;
        return false;
    }

    const uint8_t *pReadBuffer = (const uint8_t *)(m_pBuffer + iCurrSize);
    memcpy(a_pData, pReadBuffer, a_iCount);

    m_iBufferPosition += a_iCount;

    return true;
}
