#include "TimeoutQueue.h"


TimeoutQueue::TimeoutQueue()
:   m_NextTimeoutId(0)
{
    ;
}


TimeoutQueue::~TimeoutQueue()
{
    ;
}


TimeoutId TimeoutQueue::RegisterItem(ITimeoutItem *a_pItem)
{
    const TimeoutId iTimeoutId = m_NextTimeoutId;

    const uint32_t iTimeoutTimeMS = GetCurrentMilliseconds() + a_pItem->GetTimeoutMS();

    TimeoutQueueItem queueItem(iTimeoutId, iTimeoutTimeMS);
    TimeoutMapItem   mapItem  (a_pItem, iTimeoutTimeMS);

    m_TimeoutItemMap[iTimeoutId] = mapItem;

    m_TimeoutItemQueue.push(queueItem);

    m_NextTimeoutId = m_NextTimeoutId + 1;

    return iTimeoutId;
}


ITimeoutItem *TimeoutQueue::RetrieveItem(TimeoutId a_TimeoutId)
{
    TimeoutItemMap::iterator iter = m_TimeoutItemMap.find(a_TimeoutId);

    if (iter != m_TimeoutItemMap.end())
    {
        if (iter->second.m_pItem == NULL)
        {
            return NULL;
        }

        iter->second.m_TimeoutTimeMS = GetCurrentMilliseconds() + iter->second.m_pItem->GetTimeoutMS();

        return iter->second.m_pItem.get();
    }

    return NULL;
}


void TimeoutQueue::RemoveItem(TimeoutId a_TimeoutId)
{
    m_TimeoutItemMap.erase(a_TimeoutId);
}


void TimeoutQueue::UpdateTimeouts()
{
    uint32_t iCurrentTimeMS = GetCurrentMilliseconds();

    while (m_TimeoutItemQueue.size() > 0)
    {
        TimeoutQueueItem queueItem = m_TimeoutItemQueue.top();

        const TimeoutId iItemTimeoutId = queueItem.m_TimeoutId;
        const uint32_t  iItemTimeoutMS = queueItem.m_TimeoutTimeMS;

        // We can terminate once we run out of timed out items
        if (iItemTimeoutMS > iCurrentTimeMS)
        {
            break;
        }

        // Pop the item, we're either going to time it out or reinsert it
        m_TimeoutItemQueue.pop();

        TimeoutItemMap::iterator iter = m_TimeoutItemMap.find(iItemTimeoutId);

        if (iter == m_TimeoutItemMap.end())
        {
            // Item has already been deleted, just continue
            continue;
        }

        TimeoutMapItem mapItem = iter->second;

        // Check to see if the item has been refreshed since it was inserted
        if (mapItem.m_TimeoutTimeMS > iCurrentTimeMS)
        {
            TimeoutQueueItem queueItem(iItemTimeoutId, mapItem.m_TimeoutTimeMS);
            m_TimeoutItemQueue.push(queueItem);
            continue;
        }

        // By this point, the item is definitely timed out
        // Invoke the timeout function to see how to proceed
        ETimeoutResult result = mapItem.m_pItem->OnTimeout();

        if (result == ETimeoutResult_Refresh)
        {
            const uint32_t iNewTimeoutMS = iCurrentTimeMS + mapItem.m_pItem->GetTimeoutMS();
            
            // Update map entry timeout
            mapItem.m_TimeoutTimeMS = iNewTimeoutMS;

            // Re-insert into priority queue
            TimeoutQueueItem queueItem(iItemTimeoutId, iNewTimeoutMS);
            m_TimeoutItemQueue.push(queueItem);

            continue;
        }

        m_TimeoutItemMap.erase(iItemTimeoutId);
    }
}
